% Generate a grid-based lookup table approximation structure.
%
%*Copyright  2013 Universitt Osnabrck
%*This file is part of the UOSLIB,
%*
%*UOSLIB is free software; you can use it and/or modify it under the terms
%*of the GNU General Public License as published by the Free Software
%*Foundation; either version 3 of the License, or (at your option) any
%*later version.
%*
%*UOSLIB is distributed in the hope that it will be useful for studies and
%*research work on learning systems. But it comes WITHOUT ANY WARRANTY;
%*without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
%*PARTICULAR PURPOSE. See the GNU General Public License for more details.
%*
%*You should have received a copy of the GNU General Public License along
%*with this program; if not, write to the Free Software Foundation, Inc.,
%*59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
%
% This function currently only works for one or two input dimensions.
% Range of the inputs is always [-10,10]. The base functions are
% distributed linearly accross the dimension and result in a regular grid.
% On these grid nodes either triangular or gaussian radial basis functions
% are placed, resulting in different interpolations.
%
% @author Andreas Buschermhle (andbusch(at)uos.de)
% @author Jens Hlsmann (jehuelsm(at)uos.de)
% @date 2013-03-07
% @version 1.4
%
% @input num - integer > 0, number of base functions per dimension
%
% @input dim - integer > 0, number of dimensions
%
% @input base - string, base function of the grid, choose between
%
%                - 'lin' for triangular, i.e. linear interpolation
%
%                - 'gauss' for gaussian
%
% @return ILS - matlab structure describing the learning system

function ILS = icl_genGLT(num, dim, base)
if(nargin < 3), base = 'lin'; end

ILS.phi = cell(1, num^dim); %get space for the base functions

if (dim == 1)
    pos = linspace(-10,10,num);
    for i=1:num
        if(strcmp(base,'lin'))
            ILS.phi{i} = (@(x)max(0,1-abs(pos(i)-x)/(20/(num-1))));
        elseif(strcmp(base,'gauss'))
            ILS.phi{i} = (@(x)exp(-(pos(i)-x).^2/(-(10/(num-1))^2/log(0.5))));
        end
    end
elseif(dim==2)
    pos = linspace(-10,10,num);
    for i=1:num
        for j=1:num
            if(strcmp(base,'lin'))
                ILS.phi{(i-1)*num+j} = (@(x)max(0,1-abs(pos(i)-x(1))/(20/(num-1)))*max(0,1-abs(pos(j)-x(2))/(20/(num-1))));
            elseif(strcmp(base,'gauss'))
                ILS.phi{(i-1)*num+j} = (@(x)exp(-norm([pos(i) pos(j)]-x)^2/(-(10/(num-1))^2/log(0.5))));
            end
        end
    end
else
    error('Currently only allowed for up to two dimensions!');
end
